##############################################################

##        The Algorithm for Industrial Districts in R       ##
##        Released in April 2017                            ##

##############################################################


###   FOREWORD

# Before running the algorithm create a folder on your pc. The folder must contain:

# - Your input Data Set with Business Census data on total employment (at the establishment 
# level) and number of establishments aggregated
# by: LMA and economic activity, manufacturing industry and size class of the establishment 
# of the economic unit (see PPT presentation for details on the creation of these variables).
# You can use the Package data.table to calculate the aggregate data.

# - A data set containing the complete list of your Country's LMAs with names and codes

# In order to run the algorithm, please assign the following names to your input datasets and variables:
#   - your input data set:"Business_Census"
#   - persons employed: "emp"
#   - class size: "f_size"
#   - economic activity:"T1"
#   - manufacturing industry:"T2"
#   - your key variable for LMA code: "LMAKey"
#   - your the name of your key variable for LMA name: "LMAName"
#   - your data set containing the complete list of your Country's LMAs with names and codes: "ListLMAs"

# Please refer to maptools library for cartographical representation of results obtained from the application of present algorithm. 

paste("Today is", date())

###   LOADING REQUIRED PACKAGES 

install.packages("foreign")
install.packages("xlsx")
install.packages("sqldf")

library(foreign)
library(haven)
library(sqldf)
library(xlsx)
library(tcltk)

library(datatable)

# Cleaning workspace
rm(list = ls())

### FIND YOUR FOLDER PATH
getwd() 

### READ RDA OR DTA FILE OR SAS DATASET
setwd("YOUR.FOLDER.PATH")
load("Business_Census.rda")

Business_Census<- read.dta("Business_Census.dta")
ListLMAs<- read.dta("ListLMAs.dta")

Business_Census <- read_sas("YOUR.FOLDER.PATH/Business_Census.sas7bdat")
ListLMAs <- read_sas("YOUR.FOLDER.PATH/ListLMAs.sas7bdat")



################
#### STEP 1 #### 
################

### Identification of LMAs specialized in manufacturing
# Calculation of LQ and prevailing employment either in business services (05), 
# consumer services (06), manufacturing (03)


BASE1 <- sqldf("select NUM1.LMAKey, NUM1.T1, n1,d1,n2,d2, n1/d1 as NUMERATOR, n2/d2 as DENOMINATOR,
               (n1/d1) / (n2/d2) as LQ,
               ( (n1/d1) - (n2/d2) ) * d1 as prev
               from
               (select LMAKey, T1, sum(emp) as n1 
               from Business_Census group by LMAKey, T1) NUM1
               INNER JOIN (select T1, sum(emp) as d1  
               from Business_Census group by T1) DEN1 on (NUM1.T1=DEN1.T1)
               INNER JOIN (select LMAKey, sum(emp) as n2 
               from Business_Census group by LMAKey) NUM2 on (NUM1.LMAKey=NUM2.LMAKey)
               CROSS JOIN (select sum(emp) as d2 
               from Business_Census) DEN2
               where NUM1.T1 in ('03','05','06') and (n1/d1) / (n2/d2) > 1.00000000 
               order by NUM1.LMAKey, NUM1.T1
               ")

# CHECK #
sqldf("select sum(n1) from BASE1") 
sqldf("select sum(d1) from (select DISTINCT t1, d1 from BASE1)")
sqldf("select sum(n2) from (select DISTINCT LMAKey, n2 from BASE1)")
sqldf("select count (distinct LMAKey) from BASE1")
# END CHECK #


# SELECTION CRITERIA 
#Assess the prevailing (manufacturing) industry by LMA 

TEST1 <- sqldf("select LMAKey,T1,prev from BASE1 
               where T1='03' and LMAKey||prev in
               (
               select LMAKey||prev from
               (select LMAKey,max(prev) as prev from BASE1 where prev > 0 group by LMAKey) 
               )
               order by LMAKey
               ")


##################
##### STEP 2 #####
##################

### Identification of manufacturing LMAs based on MSMEs

# Calculate LQ by firm's class size in manufacturing LMAs

BASE2 <- sqldf("select NUM1.LMAKey, NUM1.f_size,n1,d1,n2,d2,n1/d1 as NUMERATOR,n2/d2 as DENOMINATOR,
               (n1/d1) / (n2/d2) as LQ
               from
               (select LMAKey, f_size, sum(emp) as n1 
               from Business_Census where T1='03' group by LMAKey, f_size) NUM1
               INNER JOIN (select f_size, sum(emp) as d1
               from Business_Census where T1='03' group by f_size) DEN1 
               on (NUM1.f_size=DEN1.f_size)
               INNER JOIN (select LMAKey, sum(emp) as n2 
               from Business_Census where T1='03' group by LMAKey) NUM2 
               on (NUM1.LMAKey=NUM2.LMAKey)
               CROSS JOIN (select sum(emp) as d2 
               from Business_Census where T1='03') DEN2
               where NUM1.LMAKey in (select LMAKey from TEST1)
               order by NUM1.LMAKey, NUM1.f_size
               ")


# CHECK;
sqldf("select sum(n1) from BASE2")
sqldf("select sum(d1) from (select DISTINCT f_size, d1 from BASE2)")
sqldf("select sum(n2) from (select DISTINCT LMAKey, n2 from BASE2)")
# END

# SELECTION CRITERIA 
#Assess the manufacturing MSMEs based LMAs by selecting those with highest LQ value and featured by enterprise class size = 1,2,3;

TEST2 <- sqldf("select LMAKey,f_size,LQ from BASE2 where f_size <= 3 and LQ > 1 and
               LMAKey||LQ in
               (
               select LMAKey||LQ from
               (select LMAKey, max(LQ) as LQ from BASE2 where LQ > 1 group by LMAKey)
               )
               order by LMAKey
               ")

# Findsmanufacturing MSMEs based LMAs



##################
##### STEP 3 ##### ;
##################

### Identification of dominant industry 

# Calculate LQ by manufacturing industry

BASE3 <- sqldf("select NUM1.LMAKey, NUM1.T2,n1,d1,n2,d2,n1/d1 as NUMERATOR,n2/d2 as DENOMINATOR,
               (n1/d1) / (n2/d2) as LQ,
               CASE when NUM1.T2 != '99' and (n1/d1) / (n2/d2) > 1 then ( (n1/d1) - (n2/d2) ) * d1 else 0 END as prev
               from
               (select LMAKey, T2, sum(emp) as n1  from Business_Census where T2 != '99' group by LMAKey, T2) NUM1
               INNER JOIN (select T2, sum(emp) as d1 from Business_Census where T2 != '99' group by T2) DEN1 on (NUM1.T2=DEN1.T2)
               INNER JOIN (select LMAKey, sum(emp) as n2 from Business_Census where T2 != '99' group by LMAKey) NUM2 on (NUM1.LMAKey=NUM2.LMAKey)
               CROSS JOIN (select sum(emp) as d2 from Business_Census where T2 != '99') DEN2
               where NUM1.LMAKey in (select LMAKey from TEST2)
               order by NUM1.LMAKey, NUM1.T2
               ")


# check;
sqldf("select sum(n1) from BASE3") 
sqldf("select sum(d1) from (select DISTINCT T2, d1 from BASE3)")
sqldf("select sum(n2) from (select DISTINCT LMAKey, n2 from BASE3)")
# end;

# SELECTION CRITERIA 
# Define LMA's dominant industry by selecting the highest prevailing employment value

TEST3 <- sqldf("select LMAKey,T2,prev from BASE3 where T2 != '99' and LMAKey||prev in
               (
               select LMAKey||prev from
               (select LMAKey,max(prev) as prev from BASE3 where LQ > 1 and prev > 0 group by LMAKey) 
               )
               order by LMAKey
               ")


##################
##### STEP 4 ##### 
##################
### Identification of LMAs based on MSMEs by verifying 4a and 4b requirements of dominant industry

# SELECTION CRITERIA
# Create a list of LMAs not satisfying 4b requirements [In the case of only one medium enterprise employment 
# has to mainly concentrate in MSMEs]


TEST4 <- sqldf("select A.LMAKey,A.T2,A.f_size, n1,d1,n1/d1 as RATIO2 from
               (
               select LMAKey,T2,f_size,sum(emp) as n1 from Business_Census  
               where (f_size <= 3 or f_size is null) and LMAKey||T2 in (select LMAKey||T2 from TEST3)
               group by LMAKey,T2,f_size
               ) A
               LEFT JOIN 
               (
               select LMAKey,T2,sum(emp) as d1 from Business_Census
               where f_size = 3 and LMAKey||T2 in (select LMAKey||T2 from TEST3)
               group by LMAKey,T2
               ) B on (A.LMAKey=B.LMAKey) and (A.T2=B.T2)
               where A.LMAKey||A.T2 in 
               (select LMAKey||T2 from Business_Census where f_size = 3 and l_units = 1) 
               and n1/d1 < 0.5
               order by A.LMAKey,A.T2,A.f_size
               ")

TEST4_l <- sqldf("select A.LMAKey,A.T2 from 
               (
               select LMAKey,t2,count(*) as n_spec from Business_Census 
               where LMAKey||T2 in (select LMAKey||T2 from TEST3)
               group by LMAKey,T2
               ) A 
               LEFT JOIN 
               (
               select LMAKey,T2,f_size from Business_Census 
               where f_size = 3 and l_units = 1
               ) B 
               on (A.LMAKey=B.LMAKey) and (A.T2=B.T2)
               where n_spec=1
               ")
# Keeps LMAs satisfying both 4a and 4b requirements 
# [More than half of total employment in dominant industry must be in MSMEs]

BASE4 <- sqldf("select A.LMAKey,A.T2,n1/d1 as rapp1
               from
               (
               select LMAKey,T2,sum(emp) as n1 from Business_Census  
               where f_size <= 3 and LMAKey||T2 in (select LMAKey||T2 from TEST3)
               group by LMAKey,T2
               ) A
               INNER JOIN 
               (
               select LMAKey,T2,sum(emp) as d1 from Business_Census  
               where LMAKey||T2 in (select LMAKey||T2 from TEST3)
               group by LMAKey,T2
               ) B on (A.LMAKey=B.LMAKey) and (A.T2=B.T2)
               where n1/d1 > 0.5 and A.LMAKey||A.T2 not in (select LMAKey||T2 from TEST4)
               and A.LMAKey||A.T2 not in (select LMAKey||T2 from TEST4_l)
               order by A.LMAKey,A.T2
               ")

# END


#    #    #    #    #

### Final dataset ###;
DISTRICTS2011_label <- sqldf("select l.LMAKey,l.T2, r.LMAName from BASE4 AS L
                             left join ListLMAs as r
                             on l.LMAKey=r.LMAKey
                             ")

DISTRICTS2011_label[,2]=as.numeric(DISTRICTS2011_label[,2]) 


DISTRICTS2011_label$t2_f = cut(DISTRICTS2011_label[,2], 
                   breaks=c("1", "2","3","4","5","6","7","8","9","10","11", Inf),
                   right=FALSE,
                   labels=c("1"="Textile and clothing",
                            "2"="Leather and footwear",
                            "3"="Households goods",
                            "4"="Jewellery, musical instruments and toys",
                            "5"="Food and beverages",
                            "6"="Machinery",
                            "7"="Manufacture of basic metals and fabricated metal products",
                            "8"="Chemicals and plastics",
                            "9"="Transport equipment",
                            "10"="Paper, publishing and printing",
                            "11"="Other manufacturing activities"), 
                   ordered_result=F)

# Export file final results
write.xlsx(TEST1,"YOUR.FOLDER.PATH/DISTRICTS2011_label.xls")


