/*****************************************************************************************
	    The Algorithm for Industrial Districts in SAS 
	    Released in Paris, 27th January 2017 
*****************************************************************************************/

/* FOREWORD
Before running the algorithm create a folder on your pc.
The folder must contain:
- YOURDATA_BusinessCensus: your input SAS Data Set with Business Census data on total 
employment (at the establishment level) and number of establishments aggregated by: 
LMA, economic activity, manufacturing industry and size class of the establishment of 
the economic unit (see PPT presentation for the creation of such variables).
- YOURCOUNTRY_ListLMAs: a SAS Data Set containing the complete list of your Country's LMAs 
with names and codes.
- YOURCOUNTRY_SHAPEFILE: Shapefiles (.shp) of your Country's LMAs must also be placed in 
the folder. Italian 2011 LMAs can be downloaded at: http://www.istat.it/en/archive/182743 
*/


/* Assigning SAS library */

%macro grabpath;
%qsubstr(%sysget(SAS_EXECFILEPATH),1,%length(%sysget(SAS_EXECFILEPATH))-%length(%sysget(SAS_EXECFILEname)))
%mend grabpath;
%let path = %grabpath;

LIBNAME LMALIB "&path";


/* Assigning SAS macro variables */
%let BusinessCensus = YOURDATA_BusinessCensus; /* Please specify the name of your input dataset */
%let ListLMAs = YOURCOUNTRY_ListLMAs; /* Please specify the name of your Country's Data Set with LMAs */
%let ShapeFile = YOURCOUNTRY_SHAPEFILE.shp; /* Please specify the name of your Country's ShapeFile */
%let LMAKey =YOURLMAKEY; /* Please specify the name of the key variable for your LMA code as specified 
in your ShapeFile, as well as your ListLMAs and BusinessCensus files */
%let LMAName = YOURLMANAME ; /* Please specify  the name of your key variable for LMA name */

proc format;
/* Table 1 - Economic Activities */
value $t1f 
	'01'='Agro-industrial activities'
	'02'='Extractive industry'
	'03'='Manufacturing'
	'04'='Construction'
	'05'='Business services'
	'06'='Consumer services'
	'07'='Social services'
	'08'='Traditional services';

/* Table 2 - Manufacturing Industries */
value $t2f
	'99'='Not manufacturing'
	'01'='Textile and clothing'
	'02'='Leather and footwear'
	'03'='Households goods'
	'04'='Jewellery, musical instruments and toys'
	'05'='Food and beverages'
	'06'='Machinery'
	'07'='Manufacture of basic metals and fabricated metal products'
	'08'='Chemicals and plastics'
	'09'='Transport equipment'
	'10'='Paper, publishing and printing'
	'11'='Other manufacturing activities';

* Enterprise size class;
value f_size
	1='Up to 9 persons employed'
	2='From 10 to 49 persons employed'
	3='From 50 to 249 persons employed'
	4='250 persons employed or more';
run;

* Descriptive statistics of the dataset;
ods html style=journal;
proc tabulate data =LMALIB.&BusinessCensus missing;
	class  t1 t2 ; 
	var emp;
	table (t1='Economic activity - Table 1' all='TOTAL'), 
	emp='Persons employed'*f=12.0
	*(t2='Manufacturing activity - Table 2' all='TOTAL');
	format t1 $t1f. t2  $t2f. ;
	title 'Persons employed by  T1 and T2';
run;
ods html close;

ods html style=journal;
proc tabulate data =LMALIB.&BusinessCensus missing;
	class  t1 f_size; 
	var emp;
	table (t1='Economic activity - Table 1' all='TOTAL'), 
	emp='Persons employed'*f=12.0
	*(f_size='Firm size class' all='TOTAL');
	format t1 $t1f. f_size f_size.;
	title 'Persons employed by Economic activity and size class';
run;
ods html close;

ods html style=journal;
proc tabulate data = LMALIB.&BusinessCensus missing;
	class  t1 f_size; 
	var l_units;
	table (t1='Economic activity - Table 1' all='TOTAL'), 
	l_units='Local units'*f=12.0
	*(f_size='Firm size class' all='TOTAL');
	format t1 $t1f. f_size f_size.;
	title 'Local units by Economic activity and size class';
run;
ods html close;


proc sql;

******************
**** STEP 1 **** ;
******************
**** Identification of LMAs specialized in manufacturing;

* Calculation of LQ and prevailing employment either in business services (05), 
consumer services (06), manufacturing (03);

create table BASE1 as
select NUM1.&LMAKey, NUM1.T1, n1,d1,n2,d2, n1/d1 as NUMERATOR, n2/d2 as DENOMINATOR,
(n1/d1) / (n2/d2) as LQ,
CASE 
when NUM1.T1 in ('03','05','06') and (n1/d1) / (n2/d2) > 1 then ( (n1/d1) - (n2/d2) ) * d1 
else 0 
END 
as prev
from
(select &LMAKey, T1, sum(emp) as n1 label 'Persons employed by LMA, economic activity' 
from LMALIB.&BusinessCensus group by &LMAKey, T1) as NUM1
INNER JOIN (select T1, sum(emp) as d1 label 'Persons employed in ITALY, by economic activity' 
from LMALIB.&BusinessCensus group by T1) as DEN1 on (NUM1.T1=DEN1.T1)
INNER JOIN (select &LMAKey, sum(emp) as n2 label 'Persons employed by LMA, ALL economic activities' 
from LMALIB.&BusinessCensus group by &LMAKey) as NUM2 on (NUM1.&LMAKey=NUM2.&LMAKey)
CROSS JOIN (select sum(emp) as d2 label 'Persons employed in ITALY, ALL economic activities' 
from LMALIB.&BusinessCensus) as DEN2
order by &LMAKey, T1
;

* SELECTION CRITERIA
Assess the prevailing (manufacturing) industry by LMA;

create table TEST1 as select &LMAKey,T1,prev from BASE1 
where T1='03' and cats(&LMAKey,prev) in
(
select cats(&LMAKey,prev) from
(select &LMAKey,max(prev) as prev from BASE1 where prev > 0 group by &LMAKey) 
)
order by &LMAKey;


******************
**** STEP 2 **** ;
******************
**** Identification of manufacturing LMAs based on MSMEs;

* Calculate LQ by firm's class size in manufacturing LMAs;

create table BASE2 as
select NUM1.&LMAKey, NUM1.f_size,n1,d1,n2,d2,n1/d1 as NUMERATOR,n2/d2 as DENOMINATOR,
(n1/d1) / (n2/d2) as LQ
from
(select &LMAKey, f_size, sum(emp) as n1 label 'Persons employed in manufacturing by LMA, class size' 
from LMALIB.&BusinessCensus where T1='03' group by &LMAKey, f_size) as NUM1
INNER JOIN (select f_size, sum(emp) as d1 'Persons employed in manufacturing in ITALY, class size' 
from LMALIB.&BusinessCensus where T1='03' group by f_size) as DEN1 
on (NUM1.f_size=DEN1.f_size)
INNER JOIN (select &LMAKey, sum(emp) as n2 'Persons employed in manufacturing by LMA, ALL class sizes' 
from LMALIB.&BusinessCensus where T1='03' group by &LMAKey) as NUM2 
on (NUM1.&LMAKey=NUM2.&LMAKey)
CROSS JOIN (select sum(emp) as d2 'Persons employed in manufacturing in ITALY, ALL class sizes' 
from LMALIB.&BusinessCensus where T1='03') as DEN2

where NUM1.&LMAKey in (select &LMAKey from TEST1)
order by &LMAKey, f_size
;


* SELECTION CRITERIA 
Assess the manufacturing MSMEs-based LMAs by selecting those with highest LQ value 
and featured by enterprise class size = 1,2,3;
create table  TEST2 as select &LMAKey,f_size,LQ from BASE2 where f_size <= 3 and LQ > 1 and
cats(&LMAKey,LQ) in
(
select cats(&LMAKey,LQ) from
(select &LMAKey, max(LQ) as LQ from BASE2 where LQ > 1 group by &LMAKey)
)
order by &LMAKey;


******************
**** STEP 3 **** ;
******************
**** Identification of dominant industry;


* Calculate LQ by manufacturing industry;

create table BASE3 as
select NUM1.&LMAKey, NUM1.T2,n1,d1,n2,d2,n1/d1 as NUMERATOR,n2/d2 as DENOMINATOR,
(n1/d1) / (n2/d2) as LQ,
CASE when NUM1.T2 NE '99' and (n1/d1) / (n2/d2) > 1 then ( (n1/d1) - (n2/d2) ) * d1 else 0 END as prev
from
(select &LMAKey, T2, sum(emp) as n1 label 'Persons employed by LMA, T2' from LMALIB.&BusinessCensus where T2 NE '99' group by &LMAKey, T2) as NUM1
INNER JOIN (select T2, sum(emp) as d1 label 'Persons employed in ITALY, T2' from LMALIB.&BusinessCensus where T2 NE '99' group by T2) as DEN1 on (NUM1.T2=DEN1.T2)
INNER JOIN (select &LMAKey, sum(emp) as n2 label 'Persons employed by LMA, ALL T2' from LMALIB.&BusinessCensus where T2 NE '99' group by &LMAKey) as NUM2 on (NUM1.&LMAKey=NUM2.&LMAKey)
CROSS JOIN (select sum(emp) as d2 label 'Persons employed in ITALY, ALL T2' from LMALIB.&BusinessCensus where T2 NE '99') as DEN2
where NUM1.&LMAKey in (select &LMAKey from TEST2)
order by &LMAKey, T2
;


* SELECTION CRITERIA 
Define LMA's dominant industry by selecting the highest prevailing employment value;

create table TEST3 as select &LMAKey,T2,prev from BASE3 where T2 NE '99' and cats(&LMAKey,prev) in
(
select cats(&LMAKey,prev) from
(select &LMAKey,max(prev) as prev from BASE3 where LQ > 1 and prev > 0 group by &LMAKey) 
)
order by &LMAKey;


******************
**** STEP 4 **** ;
******************
**** Identification of LMAs based on MSMEs by verifying 4a and 4b requirements of dominant industry;
* SELECTION CRITERIA
* Create a list of LMAs not satisfying 4b requirements [In the case of only one medium enterprise employment has to mainly concentrate in MSMEs];

create table TEST4 as 
select coalesce(A.&LMAKey,B.&LMAKey) as &LMAKey,coalesce(A.T2,B.T2) as T2,A.f_size, n1/d1 as RATIO2
from
(
select &LMAKey,T2,f_size,sum(emp) as n1 from LMALIB.&BusinessCensus  
where f_size<=2 and cats(&LMAKey,T2) in (select cats(&LMAKey,T2) from TEST3)
group by &LMAKey,T2,f_size
) A
FULL JOIN 
(
select &LMAKey,T2,sum(emp) as d1 from LMALIB.&BusinessCensus
where f_size=3 and cats(&LMAKey,T2) in (select cats(&LMAKey,T2) from TEST3)
group by &LMAKey,T2
) B on (A.&LMAKey=B.&LMAKey) and (A.T2=B.T2)
where cats(coalesce(A.&LMAKey,B.&LMAKey),coalesce(A.T2,B.T2)) in (select cats(&LMAKey,T2) from LMALIB.&BusinessCensus 
where f_size = 3 and l_units = 1)
and n1/d1 < 0.5
order by A.&LMAKey,A.T2,A.f_size
;  

create table test4_l as 
select coalesce(A.&LMAKey,B.&LMAKey) as &LMAKey,coalesce(A.T2,B.T2) as T2
from 
(select &LMAKey,t2,count(*) as n_spec 
from LMALIB.&BusinessCensus 
where cats(&LMAKey,T2) in (select cats(&LMAKey,T2) from TEST3)
group by &LMAKey,t2
) A left join 
(
select &LMAKey,T2,f_size  from LMALIB.&BusinessCensus 
where f_size = 3 and l_units = 1
) B 
on (A.&LMAKey=B.&LMAKey) and (A.T2=B.T2)
where n_spec=1
;

* Keeps LMAs satisfying both 4a and 4b requirements 
[More than half of total employment in dominant industry must be in MSMEs];

create table BASE4 as 
select A.&LMAKey,A.T2,n1/d1 as rapp1
from
(
select &LMAKey,T2,sum(emp) as n1 from LMALIB.&BusinessCensus  
where f_size<=3 and cats(&LMAKey,T2) in (select cats(&LMAKey,T2) from TEST3)
group by &LMAKey,T2
) A
INNER JOIN 
(
select &LMAKey,T2,sum(emp) as d1 from LMALIB.&BusinessCensus  
where cats(&LMAKey,T2) in (select cats(&LMAKey,T2) from TEST3)
group by &LMAKey,T2
) B on (A.&LMAKey=B.&LMAKey) and (A.T2=B.T2)
where n1/d1 > 0.5 and cats(A.&LMAKey,A.T2) not in (select cats(&LMAKey,T2) from TEST4)
and cats(A.&LMAKey,A.T2) not in (select cats(&LMAKey,T2) from test4_l)
order by A.&LMAKey,A.T2
;


/*    *    *    *    */
*** Final dataset on Industrial Districts ***;

create table LMALIB.DISTRICTS2011_label as select l.&LMAKey,l.t2, r.&LMAName 
from base4 AS L
left join LMALIB.&ListLMAs as r
on l.&LMAKey=r.&LMAKey;

quit;

ods html path="&path" body="DISTRICTS2011_label.html";
proc print data=LMALIB.DISTRICTS2011_label;
format t2 $t2f. ;
run;
ods html close;

/*    *    *    *    */


/** MAPS **/

proc mapimport out=LMALIB.shapelma datafile="&path&ShapeFile";
run;

data DISTRICTS2011_label ;
    set LMALIB.DISTRICTS2011_label ;
        dummy_DI=1;
run;

proc sql; 

create table data_map as select l.*, r.t2, r.dummy_DI from LMALIB.&ListLMAs as l 
full join DISTRICTS2011_label as r
on l.&LMAKey=r.&LMAKey;

QUIT;

data data_map;
set data_map;
if dummy_DI=. then dummy_DI=0;
if t2='' then t2='00';
run;


/* MAP 1. IDs by dominant industry */

PROC FORMAT;
value $industry

    '00'=' Other LMA'
    '01'='Textile and clothing'
    '02'='Leather and footwear'
    '03'='Households goods'
    '04'='Jewellery, toys, etc.' /* 'and musical instruments' */
    '05'='Food and beverages'
    '06'='Machinery'
    '07'='Manufacture of basic metals, etc.' /*'and fabricated metal products'*/
    '08'='Chemicals and plastics'
    '09'='Transport equipment'
    '10'='Paper, publishing and printing'
    '11'='Other manufacturing activities';
run;

goptions reset=all cback=white gunit=pct htitle=2.5 htext=2 ftitle="arial" ftext="arial"
xpixels=900 ypixels=700 DEVICE=png;
legend1 label=none shape=bar(2,2) position=(center) across=2;
pattern1 v=ms c=cxFFFFFF; *white;
pattern2 v=ms c=CX662F14; *brown;
pattern3 v=ms c=CX4D7EBF; *light blue;
pattern4 v=ms c=CXCC2B1B; *red;
pattern5 v=ms c=CX44A616; *light green;
pattern6 v=ms c=CXE5B82E; *orange;
pattern7 v=ms c=CXD974C9; *pink;
pattern8 v=ms c=CX3230B2; *blue;
pattern9 v=ms c=CXFFFF00; *yellow;
pattern10 v=ms c=CXAEADD9; *very light violet;
pattern11 v=ms c=CXBDB2BF; *grey;

ods html path="&path";
proc gmap data=data_map map=LMALIB.shapelma;
id &LMAKey;
choro t2 /coutline=grey legend=legend1;
title 'Industrial districts 2011';
label t2='Industry of specialization';
format t2 $industry.;
run; quit;
ods html close;


/* MAP 2. IDs vs OTHER LMAs */
proc format;
value dummy_DI
0='Other LMA'
1='Industrial district 2011';
run;

goptions reset=all cback=white gunit=pct htitle=3 htext=2.5 ftitle="arial" ftext="arial";
GOPTIONS xpixels=900 ypixels=700 DEVICE=png;
legend1 label=none shape=bar(3,3) position=(center ) across=2;
pattern1 v=ms c=cxFFFFFF; * red LMAs;

ods html path="&path";
proc gmap data=data_map map=LMALIB.shapesll;
id &LMAKey;
choro dummy_DI /coutline=grey legend=legend1;
title 'Industrial districts';
label dummy_DI='District';
format dummy_DI dummy_DI.;
run; quit;
ods html close;








